"use strict";
this.name = "MarketScriptInterface_Main";
this.author = "phkb";
this.description = "Script for allowing cooperation between market_scripts via the trade-goods.plist, planetinfo.plist and shipdata.plist files.";
this.licence = "CC BY-NC-SA 4.0";

this._data = null;
this._loaded = false;
this._cleanup = false;

/*
    with a market script defined for every system, and for every commodity, the only scripts that will be called are:
        tradegoods general
        system local
        station local
    the universal script will never be called - the system script blows it away
*/

//-------------------------------------------------------------------------------------------------------------
this.startUp = function() {
    this.$loadData();
}

//-------------------------------------------------------------------------------------------------------------
this.playerWillSaveGame = function() {
    this.$saveData();
}

//-------------------------------------------------------------------------------------------------------------
this.guiScreenChanged = function(to, from) {
    if (from == "GUI_SCREEN_OPTIONS" && this._cleanup) {
        if (missionVariables.MarketScriptInterface_Data) delete missionVariables.MarketScriptInterface_Data;
        this._cleanup = false;
    }
}

//-------------------------------------------------------------------------------------------------------------
this.$loadData = function() {
    // make sure we only come here once
    if (this._loaded == true) return;
    this._data = {
        "tradegoods_general": [],
        "system_local": [],
        "station_local": []
    };
    if (missionVariables.MarketScriptInterface_Data) {
        this._data = JSON.parse(missionVariables.MarketScriptInterface_Data);
        delete missionVariables.MarketScriptInterface_Data;
    }
    this._loaded = true;
}

//-------------------------------------------------------------------------------------------------------------
this.$saveData = function() {
    missionVariables.MarketScriptInterface_Data = JSON.stringify(this._data);
    this._cleanup = true;
}

//-------------------------------------------------------------------------------------------------------------
this.$validateFunctions = function() {
    var sections = Object.keys(this._data);
    var i = sections.length;
    while (i--) {
        var callbacks = this._data[sections[i]];
        var j = callbacks.length;
        while (j--) {
            var cb = callbacks[j];
            // oops! function/ws combo not found
            if (!worldScripts[cb.worldscript][cb.functionname]) {
                callbacks.splice(j, 1);
            }
        }
    }
}

//-------------------------------------------------------------------------------------------------------------
// need addition interfaces
this.$addMarketInterface = function(section, fnName, ws, priority) {
    var avail = ["tradegoods_general", "system_local", "station_local"];
    if (avail.indexOf(section) == -1) {
        log(this.name, "Exception! Invalid section defined. Must be either 'tradegoods_general', 'system_local' or 'station_local' - tried to use " + section);
        return false;
    }
    if (!worldScripts[ws][fnName]) {
        log(this.name, "Exception! WorldScript/Function not found! " + ws + "." + fnName);
        return false;
    }
    var callbacks = this._data[section];
    var i = callbacks.length;
    var found = false;
    var low_start = -1;
    while (i--) {
        if (callbacks[i].functionname == fnName && callbacks[i].worldscript == ws) found = true;
        if (callbacks[i].priority == "lowest" && low_start == -1) low_start = i;
    }
    if (found == false) {
        // add high priority item to bottom (because we will always be going through list in reverse order)
        if (priority == "highest") {
            callbacks.push({functionname:fnName, worldscript:ws, priority:priority});
        } else {
            if (low_start == -1 || priority == "lowest") 
                // put any callbacks set to "lowest" to the top of the list
                callbacks.unshift({functionname:fnName, worldscript:ws, priority: priority});
            else {
                callbacks.splice(low_start + 1, 0, {functionname:fnName, worldscript:ws, priority: priority});
            }
        }
    }
    return true;
}

//-------------------------------------------------------------------------------------------------------------
this.$process = function(section, goodDefinition, station, systemID) {
    //log("testing", "start process section " + section);
    if (!this._loaded) this.$loadData();
    var callbacks = this._data[section];
    var i = callbacks.length;
    while (i--) {
        var cb = callbacks[i];
        if (worldScripts[cb.worldscript] && worldScripts[cb.worldscript][cb.functionname]) {
            //log("testing", "calling " + cb.worldscript + "." + cb.functionname);
            goodDefinition = worldScripts[cb.worldscript][cb.functionname](goodDefinition, station, systemID);
        }
    }
    //log("testing", "end process section " + section);
    return goodDefinition;
}

// the functions below are the end-points for all our "market_script" definitions (see "*_market.js" files)
//-------------------------------------------------------------------------------------------------------------
this.$updateGeneralTradeGoodsMarket = function(goodDefinition, station, systemID) {
    goodDefinition = worldScripts.MarketScriptInterface_Main.$process("tradegoods_general", goodDefinition, station, systemID);
    return goodDefinition;
}

//-------------------------------------------------------------------------------------------------------------
this.$updateLocalSystemMarket = function(goodDefinition, station, systemID) {
    goodDefinition = worldScripts.MarketScriptInterface_Main.$process("system_local", goodDefinition, station, systemID);
    return goodDefinition;
}

//-------------------------------------------------------------------------------------------------------------
this.$updateLocalStationMarket = function(goodDefinition, station, systemID) {
    goodDefinition = worldScripts.MarketScriptInterface_Main.$process("station_local", goodDefinition, station, systemID);
    return goodDefinition;
}
